/*----------------------------------------------------------------------------
    calories - real-time, networked, cellular automaton simulation

    Copyright (c) 2008 Ge Wang
        All rights reserved.
        http://ccrma.stanford.edu/~ge/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// name: ca_custom.cpp
// desc: implementation of custom cellular automaton classes
//
// author: Ge Wang (ge@ccrma.stanford.edu)
//   date: 2008.2.19
//-----------------------------------------------------------------------------
#include "ca_custom.h"




//-----------------------------------------------------------------------------
// name: update()
// desc: game of life rules
//-----------------------------------------------------------------------------
void CA_GOL::update( const Cell * current, Cell * next )
{
    long num;

    // get num neighbors
    num = getNumNeighbors( current );

    // any live cell with fewer than two live neighbours dies
    if( current->value > 0 && num < 2 )
    {   next->value = 0; }
    // any live cell with more than three live neighbours dies
    else if( current->value > 0 && num > 3 )
    {   next->value = 0; }
    // any live cell with two or three live neighbours lives, unchanged
    else if( current->value > 0 )
    {   next->value = current->value; }
    // any dead cell with exactly three live neighbours comes to life
    else if( current->value == 0 && num == 3 )
    {   next->value = 1; }
}




//-----------------------------------------------------------------------------
// name: update()
// desc: game of bungholio rules
//-----------------------------------------------------------------------------
void CA_Cornholio::update( const Cell * current, Cell * next )
{
    long num;

    // invert the cell
    next->value = !current->value;
}



//-----------------------------------------------------------------------------
// name: update()
// desc: game of BeyonceButtHead bungholio rules
//-----------------------------------------------------------------------------
void CA_BeyonceButtHead::update( const Cell * current, Cell * next )
{
    long num;

    // get away neighbor
    const Cell * right = getCell( current, 0, 0, AWAY );

    // copy the neighor
    next->value = right->value;
}



//-----------------------------------------------------------------------------
// name: update()
// desc: game of life + bunholio left shift rules
//-----------------------------------------------------------------------------
void CA_GOB::update( const Cell * current, Cell * next )
{
    long num;

    // get num neighbors
    num = getNumNeighbors( getCell( current, RIGHT, 0, 0 ) );

    // any live cell with fewer than two live neighbours dies
    if( current->value > 0 && num < 2 )
    {   next->value = 0; }
    // any live cell with more than three live neighbours dies
    else if( current->value > 0 && num > 3 )
    {   next->value = 0; }
    // any live cell with two or three live neighbours lives, unchanged
    else if( current->value > 0 )
    {   next->value = current->value; }
    // any dead cell with exactly three live neighbours comes to life
    else if( current->value == 0 && num == 3 )
    {   next->value = 1; }
}
